@@ -2,13 +2,14 @@ import logging
 from abc import abstractmethod, ABCMeta
 from typing import Dict, Optional
 
-from twisted.internet.defer import Deferred, DeferredList, fail, succeed
+from twisted.internet.defer import Deferred, fail, succeed, inlineCallbacks
 from twisted.python import failure
 from twisted.python.failure import Failure
 
 from vortex.DeferUtil import vortexLogFailure
 from vortex.Payload import Payload
 from vortex.PayloadEndpoint import PayloadEndpoint
+from vortex.PayloadEnvelope import PayloadEnvelope
 from vortex.TupleAction import TupleActionABC
 from vortex.VortexABC import SendVortexMsgResponseCallable
 
@@ -30,7 +31,9 @@ class TupleActionProcessorDelegateABC(metaclass=ABCMeta):
 class TupleActionProcessor:
     def __init__(self, tupleActionProcessorName: str,
                  additionalFilt: Optional[Dict] = None,
-                 defaultDelegate: Optional[TupleActionProcessorDelegateABC] = None):
+                 defaultDelegate: Optional[TupleActionProcessorDelegateABC] = None,
+                 acceptOnlyFromVortex: Optional[str] = None
+                 ) -> None:
         """ Constructor
 
         :param tupleActionProcessorName: The name of this observable
@@ -38,6 +41,9 @@ class TupleActionProcessor:
         :param additionalFilt: Any additional filter keys that are required
 
         :param defaultDelegate: The default delegate to send all actions to
+
+        :param acceptOnlyFromVortex: Accept requests only from this vortex,
+            Or None to accept from any.
         """
 
         self._tupleActionProcessorName = tupleActionProcessorName
@@ -49,9 +55,10 @@ class TupleActionProcessor:
         if additionalFilt:
             self._filt.update(additionalFilt)
 
-        self._endpoint = PayloadEndpoint(self._filt, self._process)
+        self._endpoint = PayloadEndpoint(self._filt, self._process,
+                                         acceptOnlyFromVortex=acceptOnlyFromVortex)
 
-        self._tupleProcessorsByTupleName = {}
+        self._tupleProcessorsByTupleName: Dict[str, TupleActionProcessorDelegateABC] = {}
 
     def setDelegate(self, tupleName: str, processor: TupleActionProcessorDelegateABC):
         """ Add Tuple Action Processor Delegate
@@ -61,33 +68,36 @@ class TupleActionProcessor:
 
         """
         assert not tupleName in self._tupleProcessorsByTupleName, (
-            "TupleActionProcessor:%s, Tuple name %s is already registered" %
-            (self._tupleActionProcessorName, tupleName))
+                "TupleActionProcessor:%s, Tuple name %s is already registered" %
+                (self._tupleActionProcessorName, tupleName))
 
         assert isinstance(processor, TupleActionProcessorDelegateABC), (
-            "TupleActionProcessor:%s, provider must be an"
-            " instance of TupleActionProcessorDelegateABC"
-            % self._tupleActionProcessorName)
+                "TupleActionProcessor:%s, provider must be an"
+                " instance of TupleActionProcessorDelegateABC"
+                % self._tupleActionProcessorName)
 
         self._tupleProcessorsByTupleName[tupleName] = processor
 
     def shutdown(self):
         self._endpoint.shutdown()
 
-    def _process(self, payload: Payload,
+    @inlineCallbacks
+    def _process(self, payloadEnvelope: PayloadEnvelope,
                  sendResponse: SendVortexMsgResponseCallable, **kwargs):
         """ Process the Payload / Tuple Action
         """
 
+        payload = yield payloadEnvelope.decodePayloadDefer()
+
         assert len(payload.tuples) == 1, (
-            "TupleActionProcessor:%s Expected 1 tuples, received %s" % (
-                self._tupleActionProcessorName, len(payload.tuples)))
+                "TupleActionProcessor:%s Expected 1 tuples, received %s" % (
+            self._tupleActionProcessorName, len(payload.tuples)))
 
         tupleAction = payload.tuples[0]
 
         assert isinstance(tupleAction, TupleActionABC), (
-            "TupleActionProcessor:%s Expected TupleAction, received %s" % (
-                self._tupleActionProcessorName, tupleAction.__class__))
+                "TupleActionProcessor:%s Expected TupleAction, received %s" % (
+            self._tupleActionProcessorName, tupleAction.__class__))
 
         tupleName = tupleAction.tupleName()
 
@@ -96,29 +106,35 @@ class TupleActionProcessor:
             delegate = processor.processTupleAction
 
         elif self._defaultDelegate:
-            delegate =self._defaultDelegate.processTupleAction
+            delegate = self._defaultDelegate.processTupleAction
 
         else:
             raise Exception("No delegate registered for %s" % tupleName)
 
         d = self._customMaybeDeferred(delegate, tupleAction)
-        d.addCallback(self._callback, payload.filt, tupleName, sendResponse)
-        d.addErrback(self._errback, payload.filt, tupleName, sendResponse)
+        d.addCallback(self._callback, payloadEnvelope.filt, tupleName, sendResponse)
+        d.addErrback(self._errback, payloadEnvelope.filt, tupleName, sendResponse)
 
-
-    def _callback(self, result, replyFilt: dict, tupleName:str,
-                       sendResponse: SendVortexMsgResponseCallable):
+    @inlineCallbacks
+    def _callback(self, result, replyFilt: dict, tupleName: str,
+                  sendResponse: SendVortexMsgResponseCallable):
 
         if not isinstance(result, list):
             result = [result]
         payload = Payload(filt=replyFilt, tuples=result)
 
-        d = sendResponse(payload.toVortexMsg())
-        d.addErrback(lambda f: logger.error("Failed to send TupleAction response for %s",
-                                            tupleName))
+        payloadEnvelope = yield payload.makePayloadEnvelopeDefer()
+        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
+        try:
+            yield sendResponse(vortexMsg)
 
-    def _errback(self, result: Failure, replyFilt: dict,tupleName:str,
-                       sendResponse: SendVortexMsgResponseCallable):
+        except Exception as e:
+            logger.error("Failed to send TupleAction response for %s", tupleName)
+            logger.exception(e)
+
+    @inlineCallbacks
+    def _errback(self, result: Failure, replyFilt: dict, tupleName: str,
+                 sendResponse: SendVortexMsgResponseCallable):
 
         logger.error("TupleActionProcessor:%s Failed to process TupleActon",
                      self._tupleActionProcessorName)
@@ -126,12 +142,16 @@ class TupleActionProcessor:
         vortexLogFailure(result, logger)
         failureMessage = result.getErrorMessage()
 
-        payload = Payload(filt=replyFilt, result=failureMessage)
+        payloadEnvelope = PayloadEnvelope(filt=replyFilt, result=failureMessage)
+        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
+        try:
+            yield sendResponse(vortexMsg)
 
-        d = sendResponse(payload.toVortexMsg())
-        d.addErrback(lambda f: logger.error("Failed to send TupleAction response for %s\n%s",
-                                            tupleName,
-                                            failureMessage))
+        except Exception as e:
+            logger.error("Failed to send TupleAction response for %s\n%s",
+                         tupleName,
+                         failureMessage)
+            logger.exception(e)
 
     def _customMaybeDeferred(self, f, *args, **kw):
         try:
